/*
 * factory.cpp - PacketFactory - produces commonly used packet types
 *
 * Written by Michael "Mr. Sump" Poppitz.
 *
 */

 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "packet.h"
#include "factory.h"

PacketFactory::PacketFactory() {
}

PacketFactory::~PacketFactory() {
}

Packet *PacketFactory::produceSABM(char *src, int sssid, char *dest, int dssid) {
	Packet *p = new Packet();
	setBasics(p, src, sssid, dest, dssid);
	p->setFrameType(p->FRAME_U, p->UFRAME_SABM);
	return (p);
}

Packet *PacketFactory::produceDISC(char *src, int sssid, char *dest, int dssid) {
	Packet *p = new Packet();
	setBasics(p, src, sssid, dest, dssid);
	p->setFrameType(p->FRAME_U, p->UFRAME_DISC);
	return (p);
}

Packet *PacketFactory::produceUI(char *src, int sssid, char *dest, int dssid, char *msg) {
	Packet *p = new Packet();
	setBasics(p, src, sssid, dest, dssid);
	p->setFrameType(p->FRAME_U, p->UFRAME_UI);
	p->setInfo((unsigned char *)msg, strlen(msg));
	return (p);
}

Packet *PacketFactory::produceBeacon(char *src, int sssid, char *dest, int dssid, char *msg) {
	return (produceUI(src, sssid, dest, dssid, msg));
}

Packet *PacketFactory::produceI(char *src, int sssid, char *dest, int dssid, int pid, unsigned char *text, int len, int rseq, int sseq, bool cr) {
	Packet *p = new Packet();
	setBasics(p, src, sssid, dest, dssid);

	if (cr) {
		p->setCommand();
	} else {
		p->setResponse();
	}

	p->setSendSequence(sseq);
	p->setReceiveSequence(rseq);

	p->setFrameType(p->FRAME_I, pid);
	p->setInfo(text, len);

	return (p);
}

void PacketFactory::setBasics(Packet *p, char *src, int sssid, char *dest, int dssid) {
	p->setSourceCall(src, sssid);
	p->setDestinationCall(dest, dssid);
	p->setRepeaterCount(0);

	p->setCommand();
}

/**** That's all *********************************************************/
