/*
 * s2digilimit.cpp - Silencer Toolkit Example Application
 *
 * Written by Michael "Mr. Sump" Poppitz.
 *
 */

 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "trx.h"
#include "packet.h"


bool check(Packet *pa, char *call, int limit) {		/* does the testing for our criteria */
	char used[7];

	if (pa->getFrameType() == pa->FRAME_U)
		return (false);				/* we cannot do anything about unnumbered frames */

	if (pa->getRepeaterCount() <= limit)
		return (false);				/* we dont care for up to limit repeated packets */

	pa->getDestinationCall(used);			/* try destination first */
	if (!strncmp(used, call, strlen(used))) {
		return (true);				/* if match our work is done here */
	} else {
		pa->getSourceCall(used);		/* otherwise go for source call */
	}
	if (!strncmp(used, call, strlen(used))) {
		return (true);				/* if match, return success */
	} else {
		return (false);				/* otherwise return failure */
	}
}

void notify(Packet *pa) {				/* notify admin through stdout */
	char s[7], d[7];

	pa->getSourceCall(s);
	pa->getDestinationCall(d);
	printf(
		"Match: %s -> %s via %i nodes\n",
		s, d, pa->getRepeaterCount()
	);
}

void attack(TRX *trx, Packet *pa) {			/* give the desired response */
	pa->reverseCalls();				/* form a response */
	pa->setFrameType(pa->FRAME_U, pa->UFRAME_DISC);	/* which in our case is a DISC */
	trx->tx(pa);					/* transmit */
}


int main(int argc, char **argv) {

	if (argc != 3) {
		printf("Usage: %s <BOXCALL> <DIGILIMIT>\n\n", argv[0]);
		exit(EXIT_FAILURE);
	}						/* give instructions if args look bogus */

	char *call = argv[1];				/* since this is not an example on */
	int limit = argv[2][0] - 0x30;			/* user input validation this is sufficient ;) */
	printf(
		"Rule: No more than %i repeater "
		"to and from %s.\n",
		limit, call
	);						/* summarize information given */

	TRX *trx = new TRX();				/* get an trx, so we can listen for packets */
	if (trx->init() < 0) {
		perror("init"); 
		exit(EXIT_FAILURE);
	}						/* something went wrong, abort */

	while (Packet *pa = trx->rx()) {		/* keep listening until error occurs */
		if (check(pa, call, limit)) {		/* check if matches rules */
			notify(pa);			/* report violation */
			attack(trx, pa);		/* attack */
		}
		delete pa;				/* destroy packet */
	}

	return(EXIT_SUCCESS);
}
